#version 410 core

uniform float fGlobalTime; // in seconds
uniform vec2 v2Resolution; // viewport resolution (in pixels)

uniform sampler1D texFFT; // towards 0.0 is bass / lower freq, towards 1.0 is higher / treble freq
uniform sampler1D texFFTSmoothed; // this one has longer falloff and less harsh transients
uniform sampler1D texFFTIntegrated; // this is continually increasing

layout(location = 0) out vec4 out_color; // out_color must be written in order to see anything

vec4 plas( vec2 v, float time )
{
  float c = 0.5 + tan( v.x * 10.0 ) + tan( sin( time + v.y ) * 20.0 );
  return vec4( sin(c * 0.2 + cos(time)), c * 0.15, cos( c * 0.1 + time / .4 ) * .25, 1.0 );
}


float scene (vec3 p ){
    float c = 2;

  p = mod(p , c) - (c * 0.5);

    float s =  length(p) - texture(texFFTIntegrated, p.x).x;
    float g = p.y + sin(p.y * 4);

    return min(s,g);

}

vec4 trace(vec3 ro, vec3 dir ){
    
    float d = 0;
    vec3 p = ro;
    float td = 0;
   for (int i = 0 ; i < 8 ; i++){
        d = scene(p);
        p = p + dir * d;
       if (d < 0.004){
       return vec4(td/10);

    }
    return vec4(0);
}

}

void main(void)
{
  vec2 uv = (vec2(gl_FragCoord.x / v2Resolution.x, gl_FragCoord.y / v2Resolution.y));
  uv -= 0.5;
  uv /= vec2(v2Resolution.y / v2Resolution.x, 1);
  uv.x = uv.x/uv.y - texture(texFFTIntegrated, uv.x).x;
 
  vec3 co = vec3(0, sin(fGlobalTime),-1);
  vec3 ro = vec3(uv, 0);

  vec3 dir = ro + co;
  vec4 tr = trace(ro,dir);
  vec2 m;

  m.x = atan(uv.x / uv.y) / 3.14;
  m.y = 1 / length(uv) * .2;
  float d = m.y;

  float f = texture( texFFT, d ).r * 100;
  m.x += sin( fGlobalTime ) * 0.1;
  m.y += fGlobalTime * 0.25;

  vec4 t = plas( m * 3.14, fGlobalTime ) / d;
  t = clamp( t, 0.0, 1.0 );
  out_color = t+d + tr;

}